@@ -4,6 +4,7 @@ import pkg_resources
 
 from . import io, log
 from .backoff import asyncbackoff
+from .circuit_breaker import CircuitBreaker, CircuitBroken, cutout
 from .context import Context, get_context
 from .entrypoint import entrypoint
 from .iterator_wrapper import IteratorWrapper
@@ -31,7 +32,7 @@ plugins = {
 }
 
 
-def setup_plugins():
+def setup_plugins() -> None:
     logger = logging.getLogger(__name__)
     for name, plugin in plugins.items():
         try:
@@ -50,9 +51,12 @@ __all__ = (
     "awaitable",
     "bind_socket",
     "cancel_tasks",
+    "CircuitBreaker",
+    "CircuitBroken",
     "chunk_list",
     "Context",
     "context_partial",
+    "cutout",
     "entrypoint",
     "get_context",
     "io",
