@@ -1,6 +1,8 @@
 import asyncio
 from functools import wraps
-from typing import Callable, Optional, Type, TypeVar, Union
+from typing import (
+    Any, Awaitable, Callable, Optional, Tuple, Type, TypeVar, Union,
+)
 
 from .timeout import timeout
 
@@ -14,10 +16,26 @@ def asyncbackoff(
     attempt_timeout: Optional[Number],
     deadline: Optional[Number],
     pause: Number = 0,
-    *exc: Type[Exception], exceptions=(),
+    *exc: Type[Exception], exceptions: Tuple[Type[Exception], ...] = (),
     max_tries: int = None,
     giveup: Callable[[Exception], bool] = None
-):
+) -> Any:
+    """
+    Patametric decorator that ensures that ``attempt_timeout`` and
+    ``deadline`` time limits are met by decorated function.
+
+    In case of exception function will be called again with similar
+    arguments after ``pause`` seconds.
+
+    :param attempt_timeout: is maximum execution time for one
+                            execution attempt.
+    :param deadline: is maximum execution time for all execution attempts.
+    :param pause: is time gap between execution attempts.
+    :param exc: retrying when this exceptions was raised.
+    :param exceptions: similar as exc but keyword only.
+    :param max_tries: is maximum count of execution attempts (>= 1).
+    :param giveup: is a predicate function which can decide by a given
+    """
 
     exceptions = exc + tuple(exceptions)
 
@@ -41,16 +59,18 @@ def asyncbackoff(
     exceptions = tuple(exceptions) or ()
     exceptions += asyncio.TimeoutError,
 
-    def decorator(func):
+    def decorator(
+        func: Callable[..., Awaitable[T]],
+    ) -> Callable[..., Awaitable[T]]:
         if attempt_timeout is not None:
             func = timeout(attempt_timeout)(func)
 
         @wraps(func)
-        async def wrap(*args, **kwargs):
+        async def wrap(*args: Any, **kwargs: Any) -> Awaitable[T]:
             last_exc = None
             tries = 0
 
-            async def run():
+            async def run() -> Any:
                 nonlocal last_exc, tries
 
                 while True:
