@@ -1,5 +1,6 @@
 import asyncio
 import socket
+import typing as t
 from functools import partial
 
 from ..utils import OptionsType, awaitable, bind_socket
@@ -11,7 +12,7 @@ class TCPServer(SimpleServer):
 
     def __init__(
         self, address: str = None, port: int = None,
-        options: OptionsType = (), sock=None, **kwargs
+        options: OptionsType = (), sock: socket.socket = None, **kwargs: t.Any
     ):
         if not sock:
             if not (address and port):
@@ -29,31 +30,33 @@ class TCPServer(SimpleServer):
         elif not isinstance(sock, socket.socket):
             raise ValueError("sock must be socket instance")
         else:
-            self.make_socket = lambda: sock
+            self.make_socket = lambda: sock     # type: ignore
 
-        self.socket = None
+        self.socket = None      # type: t.Optional[socket.socket]
 
         super().__init__(**kwargs)
 
     async def handle_client(
         self, reader: asyncio.StreamReader,
         writer: asyncio.StreamWriter
-    ):
+    ) -> t.Any:
         raise NotImplementedError
 
     def make_client_handler(
         self, reader: asyncio.StreamReader,
         writer: asyncio.StreamWriter,
-    ):
+    ) -> t.Any:
         return self.create_task(awaitable(self.handle_client)(reader, writer))
 
-    async def start(self):
+    async def start(self) -> None:
         self.socket = self.make_socket()
         self.server = await asyncio.start_server(
             self.make_client_handler,
             sock=self.socket,
         )
 
-    async def stop(self, exc: Exception = None):
+    async def stop(self, exc: Exception = None) -> None:
         await super().stop(exc)
-        await self.server.wait_closed()
+
+        if self.server:
+            await self.server.wait_closed()
