@@ -5,21 +5,24 @@ import threading
 import time
 import typing
 import warnings
+from asyncio import AbstractEventLoop, Future, Task  # noqa
 from asyncio.events import get_event_loop
 from concurrent.futures import ThreadPoolExecutor as ThreadPoolExecutorBase
 from functools import partial, wraps
 from multiprocessing import cpu_count
 from types import MappingProxyType
-from typing import NamedTuple
+from typing import Any, Callable, NamedTuple, Optional, TypeVar  # noqa
 
 from .iterator_wrapper import IteratorWrapper
 
 
+T = typing.TypeVar("T")
+F = TypeVar("F", bound=Callable[..., Any])
+
 try:
     from queue import SimpleQueue
 except ImportError:
-    from queue import Queue as SimpleQueue
-
+    from queue import Queue as SimpleQueue  # type: ignore
 
 log = logging.getLogger(__name__)
 
@@ -31,19 +34,18 @@ class ThreadPoolException(RuntimeError):
 try:
     import contextvars
 
-    def context_partial(func, *args, **kwargs):
+    def context_partial(func: F, *args: Any, **kwargs: Any) -> Any:
         context = contextvars.copy_context()
         return partial(context.run, func, *args, **kwargs)
 
 except ImportError:
     context_partial = partial
 
-
 WorkItemBase = NamedTuple(
     "WorkItemBase", (
-        ("func", typing.Callable),
-        ("args", typing.Tuple),
-        ("kwargs", typing.Dict),
+        ("func", typing.Callable[..., Any]),
+        ("args", typing.Tuple[Any, ...]),
+        ("kwargs", typing.Dict[str, Any]),
         ("future", asyncio.Future),
         ("loop", asyncio.AbstractEventLoop),
     ),
@@ -52,7 +54,9 @@ WorkItemBase = NamedTuple(
 
 class WorkItem(WorkItemBase):
     @staticmethod
-    def set_result(future, result, exception):
+    def set_result(
+        future: Future, result: Any, exception: Exception,
+    ) -> None:
         if future.done():
             return
 
@@ -61,7 +65,7 @@ class WorkItem(WorkItemBase):
         else:
             future.set_result(result)
 
-    def __call__(self):
+    def __call__(self) -> None:
         if self.future.done():
             return
 
@@ -92,23 +96,29 @@ class ThreadPoolExecutor(ThreadPoolExecutorBase):
         "__write_lock", "__thread_events",
     )
 
-    def __init__(self, max_workers=max((cpu_count(), 4)), loop=None):
+    def __init__(
+        self, max_workers: int = max((cpu_count(), 4)),
+        loop: AbstractEventLoop = None,
+    ) -> None:
+        """"""
         if loop:
             warnings.warn(DeprecationWarning("loop argument is obsolete"))
 
-        self.__futures = set()
+        self.__futures = set()  # type: typing.Set[Future[Any]]
 
-        self.__pool = set()
-        self.__thread_events = set()
-        self.__tasks = SimpleQueue()
+        self.__thread_events = set()  # type: typing.Set[threading.Event]
+        self.__tasks = SimpleQueue()  # type: SimpleQueue[Optional[WorkItem]]
         self.__write_lock = threading.RLock()
 
+        pools = set()
         for idx in range(max_workers):
-            self.__pool.add(self._start_thread(idx))
+            pools.add(self._start_thread(idx))
 
-        self.__pool = frozenset(self.__pool)
+        self.__pool = frozenset(
+            pools,
+        )  # type: typing.FrozenSet[threading.Thread]
 
-    def _start_thread(self, idx):
+    def _start_thread(self, idx: int) -> threading.Thread:
         event = threading.Event()
         self.__thread_events.add(event)
 
@@ -122,7 +132,7 @@ class ThreadPoolExecutor(ThreadPoolExecutorBase):
         thread.start()
         return thread
 
-    def _in_thread(self, event: threading.Event):
+    def _in_thread(self, event: threading.Event) -> None:
         while True:
             work_item = self.__tasks.get()
 
@@ -145,14 +155,19 @@ class ThreadPoolExecutor(ThreadPoolExecutorBase):
 
         event.set()
 
-    def submit(self, fn, *args, **kwargs):
+    def submit(  # type: ignore
+        self, fn: F, *args: Any, **kwargs: Any
+    ) -> Future:
+        """
+        Submit blocking function to the pool
+        """
         if fn is None or not callable(fn):
             raise ValueError("First argument must be callable")
 
         loop = asyncio.get_event_loop()
 
         with self.__write_lock:
-            future = loop.create_future()     # type: asyncio.Future
+            future = loop.create_future()  # type: asyncio.Future[Any]
             self.__futures.add(future)
             future.add_done_callback(self.__futures.remove)
 
@@ -168,7 +183,7 @@ class ThreadPoolExecutor(ThreadPoolExecutorBase):
 
             return future
 
-    def shutdown(self, wait=True):
+    def shutdown(self, wait: bool = True) -> None:
         for _ in self.__pool:
             self.__tasks.put_nowait(None)
 
@@ -181,26 +196,27 @@ class ThreadPoolExecutor(ThreadPoolExecutorBase):
         while not all(e.is_set() for e in self.__thread_events):
             time.sleep(0)
 
-    def _adjust_thread_count(self):
+    def _adjust_thread_count(self) -> None:
         raise NotImplementedError
 
-    def __del__(self):
+    def __del__(self) -> None:
         self.shutdown()
 
 
 def run_in_executor(
-    func, executor=None, args=(),
-    kwargs=MappingProxyType({}),
-) -> asyncio.Future:
-
+    func: F,
+    executor: ThreadPoolExecutorBase = None,
+    args: Any = (),
+    kwargs: Any = MappingProxyType({}),
+) -> Future:
     loop = get_event_loop()
     # noinspection PyTypeChecker
-    return loop.run_in_executor(
+    return loop.run_in_executor(  # type: ignore
         executor, context_partial(func, *args, **kwargs),
     )
 
 
-async def _awaiter(future):
+async def _awaiter(future: Future) -> T:
     try:
         result = await future
         return result
@@ -210,7 +226,7 @@ async def _awaiter(future):
         raise
 
 
-def threaded(func):
+def threaded(func: F) -> Callable[..., typing.Awaitable[Any]]:
     if asyncio.iscoroutinefunction(func):
         raise TypeError("Can not wrap coroutine")
 
@@ -218,35 +234,38 @@ def threaded(func):
         return threaded_iterable(func)
 
     @wraps(func)
-    def wrap(*args, **kwargs):
+    def wrap(*args: Any, **kwargs: Any) -> typing.Awaitable[Any]:
         future = run_in_executor(func=func, args=args, kwargs=kwargs)
-        result = _awaiter(future)
+        result = _awaiter(future)  # type: Any
         return result
 
     return wrap
 
 
 def run_in_new_thread(
-    func, args=(), kwargs=MappingProxyType({}),
-    detouch=True, no_return=False,
+    func: F,
+    args: Any = (),
+    kwargs: Any = MappingProxyType({}),
+    detouch: bool = True,
+    no_return: bool = False,
 ) -> asyncio.Future:
     loop = asyncio.get_event_loop()
     future = loop.create_future()
 
-    def set_result(result):
+    def set_result(result: Any) -> None:
         if future.done() or loop.is_closed():
             return
 
         future.set_result(result)
 
-    def set_exception(exc):
+    def set_exception(exc: Exception) -> None:
         if future.done() or loop.is_closed():
             return
 
         future.set_exception(exc)
 
     @wraps(func)
-    def in_thread(target):
+    def in_thread(target: F) -> None:
         try:
             loop.call_soon_threadsafe(
                 set_result, target(),
@@ -274,7 +293,10 @@ def run_in_new_thread(
     return future
 
 
-def threaded_separate(func, detouch=True):
+def threaded_separate(
+    func: F,
+    detouch: bool = True,
+) -> Callable[..., typing.Awaitable[Any]]:
     if isinstance(func, bool):
         return partial(threaded_separate, detouch=detouch)
 
@@ -282,7 +304,7 @@ def threaded_separate(func, detouch=True):
         raise TypeError("Can not wrap coroutine")
 
     @wraps(func)
-    def wrap(*args, **kwargs):
+    def wrap(*args: Any, **kwargs: Any) -> Any:
         future = run_in_new_thread(
             func, args=args, kwargs=kwargs, detouch=detouch,
         )
@@ -292,16 +314,19 @@ def threaded_separate(func, detouch=True):
     return wrap
 
 
-def threaded_iterable(func=None, max_size: int = 0):
+def threaded_iterable(
+    func: F = None,
+    max_size: int = 0,
+) -> Any:
     if isinstance(func, int):
         return partial(threaded_iterable, max_size=func)
     if func is None:
         return partial(threaded_iterable, max_size=max_size)
 
     @wraps(func)
-    def wrap(*args, **kwargs):
+    def wrap(*args: Any, **kwargs: Any) -> Any:
         return IteratorWrapper(
-            context_partial(func, *args, **kwargs),
+            context_partial(func, *args, **kwargs),  # type: ignore
             max_size=max_size,
         )
 
@@ -309,21 +334,20 @@ def threaded_iterable(func=None, max_size: int = 0):
 
 
 class IteratorWrapperSeparate(IteratorWrapper):
-    @threaded_separate
-    def _run(self):
-        return self._in_thread()
+    async def _run(self) -> Any:
+        return await run_in_new_thread(self._in_thread)
 
 
-def threaded_iterable_separate(func=None, max_size: int = 0):
+def threaded_iterable_separate(func: F = None, max_size: int = 0) -> Any:
     if isinstance(func, int):
         return partial(threaded_iterable, max_size=func)
     if func is None:
         return partial(threaded_iterable, max_size=max_size)
 
     @wraps(func)
-    def wrap(*args, **kwargs):
+    def wrap(*args: Any, **kwargs: Any) -> Any:
         return IteratorWrapperSeparate(
-            context_partial(func, *args, **kwargs),
+            context_partial(func, *args, **kwargs),  # type: ignore
             max_size=max_size,
         )
 
@@ -332,36 +356,46 @@ def threaded_iterable_separate(func=None, max_size: int = 0):
 
 class CoroutineWaiter:
     def __init__(
-        self, loop: asyncio.AbstractEventLoop, coroutine_func,
-        *args, **kwargs
+        self,
+        loop: asyncio.AbstractEventLoop,
+        coroutine_func: F,
+        *args: Any,
+        **kwargs: Any
     ):
-        self.__func = partial(coroutine_func, *args, **kwargs)
+        self.__func = partial(
+            coroutine_func, *args, **kwargs
+        )  # type: partial[Any]
         self.__loop = loop
         self.__event = threading.Event()
         self.__result = None
-        self.__exception = None
+        self.__exception = None  # type: Optional[BaseException]
 
-    def _on_result(self, task: asyncio.Task):
+    def _on_result(self, task: asyncio.Task) -> None:
         self.__exception = task.exception()
         if self.__exception is None:
             self.__result = task.result()
         self.__event.set()
 
-    def _awaiter(self):
-        task = self.__loop.create_task(self.__func())
+    def _awaiter(self) -> None:
+        task = self.__loop.create_task(self.__func())  # type: Task[Any]
         task.add_done_callback(self._on_result)
 
-    def start(self):
+    def start(self) -> None:
         self.__loop.call_soon_threadsafe(self._awaiter)
 
-    def wait(self):
+    def wait(self) -> Any:
         self.__event.wait()
         if self.__exception is not None:
             raise self.__exception
         return self.__result
 
 
-def sync_wait_coroutine(loop, coro_func, *args, **kwargs):
+def sync_wait_coroutine(
+    loop: AbstractEventLoop,
+    coro_func: F,
+    *args: Any,
+    **kwargs: Any
+) -> Any:
     waiter = CoroutineWaiter(loop, coro_func, *args, **kwargs)
     waiter.start()
     return waiter.wait()
